from easy_response.utils import is_geo_value, is_model_instance, is_valuesset
from easy_response.utils import is_related_manager, is_queryset, is_iter, is_generator

def basic_serialization(value):
    try:
        return unicode(value)
    except StopIteration:
        return repr(value)
    
def careful_serialization(value):
    if is_geo_value(value):
        return value.geojson  
    if is_model_instance(value):
        return {f.name: getattr(value, f.name) for f in value._meta.fields}
    if is_valuesset(value):
        return [dict(i) for i in value]
    if is_related_manager(value):
        return [i for i in value.all()]
    if is_iter(value): # or is_queryset or is_generator
        return [i for i in value]
    return unicode(value)



from .utils.process import to_http

class EasyResponseMiddleware(object):
    def process_view(self, request, view, args, kwargs):
        response = view(request, *args, **kwargs)
        return to_http( request, response )



from .utils.process import to_http
from .consts import BASIC_SERIALIZATION

def serialization(basic=BASIC_SERIALIZATION):
    def decorator(view):
        def wrapper(request, *args, **kwargs):
            response = view(request, *args, **kwargs)
            return to_http(request, response, basic_serialization=basic)
        return wrapper
    return decorator




from django.conf import settings

DEFAULT_STATUS_CODE = getattr(settings, 'DEFAULT_STATUS_CODE', 200)
BASIC_SERIALIZATION = getattr(settings, 'BASIC_SERIALIZATION', False)

IN_DEBUG_MODE = getattr(settings, 'DEBUG', False) and\
                'debug_toolbar' in settings.INSTALLED_APPS and\
                hasattr(settings, 'DEBUG_TOOLBAR_CONFIG')



from django.db.models.query import QuerySet, ValuesQuerySet
from django.db import models
from django.http import HttpResponse
from types import NoneType
from inspect import isgenerator, isgeneratorfunction

try:
    from django.contrib.gis.geos import GEOSGeometry
    is_geodj_project = True
except ImportError:
    import sys
    sys.stdout.write("Warning: Could not find the GEOS library.\n")
    is_geodj_project = False

is_float        = lambda x: isinstance(x, float)
is_int          = lambda x: isinstance(x, int) and not isinstance(x, bool)
is_str          = lambda x: isinstance(x, str)
is_unicode      = lambda x: isinstance(x, unicode)
is_tuple        = lambda x: isinstance(x, tuple)
is_dict         = lambda x: isinstance(x, dict)
is_list         = lambda x: isinstance(x, list)
is_bool         = lambda x: isinstance(x, bool)
is_file         = lambda x: isinstance(x, file)
is_none         = lambda x: isinstance(x, NoneType)
is_iter         = lambda x: hasattr(x, '__iter__')
is_generator    = lambda x: isgenerator(x) or isgeneratorfunction(x)
is_geo_value    = lambda x: is_geodj_project and isinstance(x, GEOSGeometry)

is_queryset        = lambda x: isinstance(x, QuerySet)
is_valuesset       = lambda x: isinstance(x, ValuesQuerySet)
is_model_instance  = lambda x: isinstance(x, models.Model)
is_http_response   = lambda x: isinstance(x, HttpResponse)
is_related_manager = lambda x: all(i in str(type(x)) for i in ["django.", "Related"])



from django.conf import settings
from django.http import HttpResponse
from .consts import IN_DEBUG_MODE, BASIC_SERIALIZATION
from .utils.serialize import basic_serialization, careful_serialization

import json
import mimetypes


class FileResponse(HttpResponse):
    """
    Http Response with file content
    """
    def __init__(self, content='', status=None, filename=None):
        content_type = mimetypes.guess_type(content.name)[0]
        filename = filename or content.name
        HttpResponse.__init__(self, content      = content, 
                                    status       = status, 
                                    content_type = content_type,)
        self['Content-Disposition'] = 'attachment; filename=' + filename


class JSONResponse(HttpResponse):
    """
    Http Response with Json content type
    """
    def __init__(self, content='', status=None, basic=BASIC_SERIALIZATION):
        content_type = 'application/json; charset=utf-8'
        if basic:
            content = json.dumps( content, default=basic_serialization, ensure_ascii=False )
        else:
            content = json.dumps( content, default=careful_serialization, ensure_ascii=False )
        HttpResponse.__init__(self, content      = content, 
                                    status       = status, 
                                    content_type = content_type,)


class JSONPResponse(HttpResponse):
    """
    Http Response with Jsonp content type
    """
    def __init__(self, content='', status=None, callback='callback', basic=BASIC_SERIALIZATION):
        content_type = 'application/javascript; charset=utf-8'
        if basic:
            content = json.dumps( content, default=basic_serialization, ensure_ascii=False )
        else:
            content = json.dumps( content, default=careful_serialization, ensure_ascii=False )
        HttpResponse.__init__(self, content      = "%s(%s)" %(callback, content),
                                    status       = status, 
                                    content_type = content_type,)


class _DebugResponse(type):
    """
    Json Response for debug purposes (django-debug-toolbar)
    """
    def __call__(self, content='', status=None, context=None, basic=BASIC_SERIALIZATION):
        if basic:
            content = json.dumps( content, default=basic_serialization, ensure_ascii=False )
        else:
            content = json.dumps( content, default=careful_serialization, ensure_ascii=False )
        return HttpResponse(content = self.__to_html(content),
                            status  = status, )

    def __to_html(self, content):
        TAG = settings.DEBUG_TOOLBAR_CONFIG.get('TAG', 'body') 
        return "<%s>%s</%s>" %(TAG, content, TAG)


class _JsonResponse(type):
    """
    HTTP Response context-dependent for Json or Jsonp content type
    """
    def __call__(self, content='', status=None, context=None, basic=BASIC_SERIALIZATION):
        callback = context.GET.get('callback', None) if context else None
        if callback:
            return JSONPResponse(content, status, callback, basic=basic)
        return JSONResponse(content, status, basic=basic)


class JsonResponse(HttpResponse):
    __metaclass__ = _DebugResponse if IN_DEBUG_MODE else _JsonResponse

